@@ -15,6 +15,8 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import os
+import subprocess
 from gi.repository import Gtk, Gdk, Gio, GLib, Adw, GObject
 from urllib.parse import unquote
 from pathlib import Path
@@ -42,6 +44,7 @@ class CurtailWindow(Adw.ApplicationWindow):
 
     headerbar = Gtk.Template.Child()
     window_title = Gtk.Template.Child()
+    toast_overlay = Gtk.Template.Child()
     filechooser_button_headerbar = Gtk.Template.Child()
     clear_button_headerbar = Gtk.Template.Child()
     menu_button = Gtk.Template.Child()
@@ -92,6 +95,12 @@ class CurtailWindow(Adw.ApplicationWindow):
         # Results
         self.listbox.bind_model(self.results_model, self.create_result_row)
 
+        # Right click on results
+        gesture = Gtk.GestureClick.new()
+        gesture.set_button(Gdk.BUTTON_SECONDARY)
+        gesture.connect("pressed", self.on_results_right_click, self.listbox)
+        self.listbox.add_controller(gesture)
+
     def create_simple_action(self, action_name, callback, shortcut=None):
         action = Gio.SimpleAction.new(action_name, None)
         action.connect('activate', callback)
@@ -129,11 +138,13 @@ class CurtailWindow(Adw.ApplicationWindow):
     def update_result_item(self, result_item, error, error_message):
         result_item.running = False
         result_item.error = error
-        if not error:
+        if error:
+            result_item.subtitle_label = error_message
+        elif result_item.skipped:
+            result_item.savings = _("Skipped")
+        else:
             result_item.savings = str(round(100 - (result_item.new_size * 100 / result_item.size), 2)) + '%'
             result_item.subtitle_label += ' → ' + sizeof_fmt(result_item.new_size)
-        else:
-            result_item.subtitle_label = error_message
 
     def create_result_row(self, result_item):
         row = Adw.ActionRow()
@@ -146,12 +157,34 @@ class CurtailWindow(Adw.ApplicationWindow):
             if image:
                 row.add_prefix(image)
 
+        skipped_info_button = Gtk.MenuButton()
+        skipped_info_button.set_valign(Gtk.Align.CENTER)
+        skipped_info_button.set_tooltip_text(_("More Information"))
+        skipped_info_button.set_icon_name("info-outline-symbolic")
+        skipped_info_button.add_css_class("flat")
+        skipped_info_button.set_visible(False)
+
+        popover = Gtk.Popover()
+        popover_label = Gtk.Label()
+        popover_label.set_text(_("Compression was skipped because compressing the file would have resulted in a larger file size."))
+        popover_label.set_halign(Gtk.Align.CENTER)
+        popover_label.set_valign(Gtk.Align.CENTER)
+        popover_label.set_margin_bottom(6)
+        popover_label.set_margin_start(6)
+        popover_label.set_margin_end(6)
+        popover_label.set_margin_top(6)
+        popover_label.set_max_width_chars(50)
+        popover_label.set_wrap(True)
+        popover.set_child(popover_label)
+
+        skipped_info_button.set_popover(popover)
+        row.add_suffix(skipped_info_button)
+
         savings_widget = Gtk.Label()
         savings_widget.add_css_class('success')
         row.add_suffix(savings_widget)
 
-        spinner = Gtk.Spinner()
-        spinner.start()
+        spinner = Adw.Spinner()
         row.add_suffix(spinner)
 
         error_image = Gtk.Image.new_from_icon_name('x-circular-symbolic')
@@ -165,17 +198,67 @@ class CurtailWindow(Adw.ApplicationWindow):
             GObject.BindingFlags.DEFAULT)
         result_item.bind_property('running', spinner, 'visible',
             GObject.BindingFlags.DEFAULT)
+        result_item.bind_property('skipped', skipped_info_button, 'visible',
+            GObject.BindingFlags.DEFAULT)
         result_item.bind_property('error', error_image, 'visible',
             GObject.BindingFlags.DEFAULT)
 
         return row
 
+    def on_results_right_click(self, gesture, button, x, y, user_data):
+        row = self.listbox.get_row_at_y(y)
+        filename = row.get_tooltip_text()
+        if not os.path.exists(filename):
+            return
+
+        popover = Gtk.PopoverMenu()
+
+        # Create actions for the popover
+        action_group = Gio.SimpleActionGroup.new()
+
+        open_image_action = Gio.SimpleAction.new("open-image", None)
+        open_image_action.connect("activate", self.on_open_image_action, filename)
+        action_group.add_action(open_image_action)
+
+        open_folder_action = Gio.SimpleAction.new("open-folder", None)
+        open_folder_action.connect("activate", self.on_open_folder_action, filename)
+        action_group.add_action(open_folder_action)
+
+        # Insert the actions into the window's action group
+        self.insert_action_group("context-menu", action_group)
+
+        # Define the structure of the popover menu
+        menu_model = Gio.Menu.new()
+        menu_model.append(_("Open Image"), "context-menu.open-image")
+        menu_model.append(_("Show in Folder"), "context-menu.open-folder")
+
+        # Set the menu model to the popover
+        popover.set_menu_model(menu_model)
+        popover.set_pointing_to(Gdk.Rectangle(x, y, 1, 1))
+        popover.set_offset(x, y)
+        popover.set_has_arrow(False)
+        popover.set_parent(self.listbox)
+        popover.popup()
+
+    def on_open_image_action(self, action, param, filename):
+        subprocess.run(['xdg-open', filename])
+
+    def on_open_folder_action(self, action, param, filename):
+        folder_path = os.path.dirname(filename)
+        subprocess.run(['xdg-open', folder_path])
+
     def set_saving_subtitle(self, new_file=None):
         if new_file is None:
             new_file = self._settings.get_boolean('new-file')
         if new_file:
-            label = _("Safe mode with '{}' suffix")\
-                                 .format(self._settings.get_string('suffix'))
+            suffix_prefix = self._settings.get_string('suffix-prefix')
+
+            if self._settings.get_int('naming-mode') == 0:
+                label = _(f"Safe mode with “{suffix_prefix}” suffix")\
+                        .format(suffix_prefix)
+            else:
+                label = _(f"Safe mode with “{suffix_prefix}” prefix")\
+                        .format(suffix_prefix)
         else:
             label = _("Overwrite mode")
         self.window_title.set_subtitle(label)
@@ -287,7 +370,7 @@ class CurtailWindow(Adw.ApplicationWindow):
             filename = filename.strip('\r\n\x00')  # remove spaces
         return filename
 
-    def check_extension(self, filename):
+    def check_format(self, filename):
         file_type = get_file_type(filename)
         if file_type:
             return file_type in ('png', 'jpg', 'webp', 'svg')
@@ -295,11 +378,19 @@ class CurtailWindow(Adw.ApplicationWindow):
             return False
 
     def create_new_filename(self, path):
+        parent = path.parents[0]
+        stem = path.stem
+        suffix_prefix = self._settings.get_string('suffix-prefix')
+        extension = path.suffix
+
         # Use new file or not
         if self._settings.get_boolean('new-file'):
-            new_filename = '{}/{}{}{}'.format(path.parents[0],
-                path.stem, self._settings.get_string('suffix'),
-                path.suffix)
+            if self._settings.get_int('naming-mode') == 0: # Suffix selected
+                new_filename = f'{parent}/{stem}{suffix_prefix}{extension}'\
+                    .format(parent, stem, suffix_prefix, extension)
+            else: # Prefix selected
+                new_filename = f'{parent}/{suffix_prefix}{stem}{extension}'\
+                    .format(parent, suffix_prefix, stem, extension)
         else :
             new_filename = str(path)
         return new_filename
@@ -307,6 +398,11 @@ class CurtailWindow(Adw.ApplicationWindow):
     def compress_filenames(self, filenames):
         filenames = self.handle_filenames(filenames)
 
+        # No files found
+        if not filenames:
+            self.toast_overlay.add_toast(Adw.Toast(title=_("No files found")))
+            return
+
         result_items = []
         for filename in filenames:
             error_message = False
@@ -318,7 +414,7 @@ class CurtailWindow(Adw.ApplicationWindow):
 
             # Check format
             size = path.stat().st_size
-            if not self.check_extension(filename) or size <= 0:
+            if not self.check_format(filename) or size <= 0:
                 error_message = _("Format of this file is not supported.").format(path.name)
 
             if not error_message:
@@ -345,12 +441,13 @@ class CurtailWindow(Adw.ApplicationWindow):
         compressor = Compressor(result_items, self.update_result_item, self.enable_compression)
         GLib.idle_add(compressor.compress_images)
 
-    def on_lossy_changed(self, switch, state):
-        self._settings.set_boolean('lossy', switch.get_active())
+    def on_lossy_changed(self, toggle, state):
+        self._settings.set_boolean("lossy", toggle.get_active_name() == "lossy")
 
     def banner_change_mode(self, *args):
         self._settings.set_boolean('new-file', True)
         self.show_warning_banner()
+        self.set_saving_subtitle()
 
     def on_preferences(self, *args):
         if self.prefs_dialog is not None:
@@ -366,7 +463,7 @@ class CurtailWindow(Adw.ApplicationWindow):
                     license_type=Gtk.License.GPL_3_0,
                     website='https://github.com/Huluti/Curtail',
                     issue_url='https://github.com/Huluti/Curtail/issues/new',
-                    version='1.10.0',
+                    version='1.13.0',
                     developers=[
                         'Hugo Posnic https://github.com/Huluti'
                     ],
@@ -385,7 +482,8 @@ class CurtailWindow(Adw.ApplicationWindow):
                 'Balló György',
                 'olokelo',
                 'Archisman Panigrahi',
-                'Maximiliano'
+                'Maximiliano',
+                'ARAKHNID'
             ]
         )
         about.set_debug_info(debug_infos())
@@ -393,3 +491,4 @@ class CurtailWindow(Adw.ApplicationWindow):
 
     def on_quit(self, *args):
         self.app.quit()
+
